from random import randint, seed, shuffle
from string import ascii_lowercase as abc
from math import sqrt
seed(47)

def save(fname, K, T, A):
    print(fname)
    N = len(A)
    f = open(fname,'w')
    print( N, K, T, file=f )
    print( *A, file=f )
    f.close()

def logrand(lo, hi):
    while True:
        b = randint(0, 30)
        if b == 0:
            x = 0
        else:
            x = randint(2**(b-1), (2**b)-1)
        if lo <= x <= hi: return x

def almost(MAXN): return randint( int(0.9*MAXN), MAXN )

def random_array(N, minval=1, maxval=10**9):
    return [ randint(minval, maxval) for _ in range(N) ]

def narrow_random_array(N):
    lo = randint(100, 10**8)
    hi = lo+logrand(10, 1000)
    return random_array(N, lo, hi)

def apply_small_edits(A):
    N = len(A)
    B = A[:]
    edit_count = logrand(1, (N+3)//4)
    for e in range(edit_count):
        i = randint(0, N-1)
        d = [-1,1][randint(0,1)] * logrand(1,100)
        B[i] = max(1, min(10**9, B[i]+d ) )
    return B

def apply_large_edits(A):
    N = len(A)
    B = A[:]
    edit_count = logrand(1, (N+3)//4)
    for e in range(edit_count):
        i = randint(0, N-1)
        B[i] = randint(1, 10**9)
    return B

def get_bounds(N, K):
    lo = 1
    hi = 1 + (N-1) * K
    if K < 0: lo, hi = hi, lo
    if hi <= 10**9:
        leeway = 10**9 - hi
        offset = randint(0, leeway)
        lo, hi = lo+offset, hi+offset
    else:
        leeway = hi - 10**9
        offset = randint(0, leeway)
        lo, hi = lo-offset, hi-offset
    if K < 0: lo, hi = hi, lo
    return lo, hi

def k_constant(N, K):
    lo, hi = get_bounds(N, K)
    return [ max(1,min(10**9,lo+i*K)) for i in range(N) ]

def around_beam(N, K, spread=100):
    lo, hi = get_bounds(N, K)
    return [ max(1,min(10**9,lo+i*K+randint(-spread,spread))) for i in range(N) ]

def sqrt_up(N, spread=7):
    B = int(sqrt(N))
    BS = [ B for _ in range(B) ]
    for i in range(N - B*B): BS[i%B] += 1
    where = 10
    A = []
    for x in BS:
        A.extend( reversed( range(where, where+10*x, 10) ) )
        where += 10*x
    return [ x + randint(-spread,spread) for x in A ]

def sqrt_down(N, spread=7):
    B = int(sqrt(N))
    BS = [ B for _ in range(B) ]
    for i in range(N - B*B): BS[i%B] += 1
    where = 10*(N+1)
    A = []
    for x in BS:
        A.extend( range(where - 10*x, where, 10) )
        where -= 10*x
    return [ x + randint(-spread,spread) for x in A ]

# \item V sade 1 aj 6 platí $n\leq 10$.
# \item V sade 2 aj 7 platí $n\leq 500$ a $k = 0$.
# \item V sade 3 aj 8 platí $n\leq 200\,000$ a $k = 0$.
# \item V sade 4 aj 9 platí $n\leq 500$.
# \item V sade 5 aj 10 platí $n\leq 200\,000$.

LIMITS = { 1: 10, 2: 500, 3: 200000, 4: 500, 5: 200000 }

for sada in [1, 4, 5]:
    MAXN = LIMITS[sada]
    TESTS = [ ]
    TESTS.append( (logrand(1,10**6), narrow_random_array(MAXN) ) )
    TESTS.append( (logrand(1,10**6), narrow_random_array(almost(MAXN)) ) )
    TESTS.append( (logrand(1,10**6), random_array(MAXN) ) )
    TESTS.append( (logrand(1,10**6), random_array(almost(MAXN)) ) )
    TESTS.append( (logrand(1,10**6), random_array(almost(MAXN)) ) )
    TESTS.append( (1, k_constant(almost(MAXN), 1)) )
    TESTS.append( (47, k_constant(almost(MAXN), 47)) )
    TESTS.append( (47, apply_small_edits(k_constant(almost(MAXN), 47))) )
    TESTS.append( (47, apply_large_edits(k_constant(almost(MAXN), 47))) )
    TESTS.append( (987654, around_beam(almost(MAXN), 987654, 100)) )
    TESTS.append( (10, around_beam(almost(MAXN), 10, 100000)) )
    TESTS.append( (10, around_beam(almost(MAXN), 10, 3)) )
    TESTS.append( (10, around_beam(almost(MAXN), 9)) )
    TESTS.append( (10, apply_small_edits(around_beam(almost(MAXN), 8, 2))) )
    TESTS.append( (10, apply_large_edits(around_beam(almost(MAXN), 8, 2))) )
    TESTS.append( (10, around_beam(almost(MAXN), 9, 7)) )
    TESTS.append( (10, around_beam(almost(MAXN), 17, 3)) )
    TESTS.append( (10, around_beam(almost(MAXN), 9, 2)) )
    TESTS.append( (10, around_beam(almost(MAXN), 9, 1)) )
    TESTS.append( (7, around_beam(almost(MAXN), -1, 20)) )
    TESTS.append( (18, sqrt_up(almost(MAXN))) )
    TESTS.append( (18, sqrt_down(almost(MAXN))) )
    TESTS.append( (18, [ x+10*i for i,x in enumerate(sqrt_up(almost(MAXN)))]) )
    TESTS.append( (18, [ x+10*i for i,x in enumerate(sqrt_down(almost(MAXN)))]) )
    if sada == 1: TESTS.append( (13, [47]) )
    for tid, test in enumerate(TESTS):
        K, A = test
        save(f'{sada:02d}.{abc[tid]}.in', K, 1, A)
        save(f'{sada+5:02d}.{abc[tid]}.in', K, 2, A)

for sada in [2, 3]:
    MAXN = LIMITS[sada]
    TESTS = [ ]
    TESTS.append( (0, narrow_random_array(MAXN) ) )
    for i in range(2):
        TESTS.append( (0, narrow_random_array(MAXN) ) )
    TESTS.append( (0, random_array(MAXN) ) )
    for i in range(3):
        TESTS.append( (0, random_array(almost(MAXN)) ) )
    TESTS.append( (0, k_constant(almost(MAXN), 0)) )
    TESTS.append( (0, k_constant(almost(MAXN), 1)) )
    TESTS.append( (0, k_constant(almost(MAXN), -1)) )
    TESTS.append( (0, apply_small_edits(k_constant(almost(MAXN), 0))) )
    TESTS.append( (0, apply_large_edits(k_constant(almost(MAXN), 0))) )
    for i in range(5):
        TESTS.append( (0, around_beam(almost(MAXN), 0, logrand(1,10**6))) )
    TESTS.append( (0, around_beam(almost(MAXN), 1)) )
    TESTS.append( (0, around_beam(almost(MAXN), -1)) )
    TESTS.append( (0, [ x+10*i for i,x in enumerate(sqrt_up(almost(MAXN),0))]) )
    TESTS.append( (0, [ x+10*i for i,x in enumerate(sqrt_down(almost(MAXN),0))]) )
    for tid, test in enumerate(TESTS):
        K, A = test
        save(f'{sada:02d}.{abc[tid]}.in', K, 1, A)
        save(f'{sada+5:02d}.{abc[tid]}.in', K, 2, A)

